﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

using IrrlichtLime;
using IrrlichtLime.Core;
using IrrlichtLime.Video;
using IrrlichtLime.Scene;
using IrrlichtLime.GUI;
using System.Runtime.InteropServices;

namespace CapstoneProjectCSharp
{
    class Guard : PlayerBase
    {
        private List<Waypoint> _Waypath;
        private bool _IsFollowingWaypath = false;
        private int _WaypathNode = 0;
        private int _WaypathInterval = 0;
        private SceneNode _LosNode;
        private bool _AllowMovement = true;



        public bool AllowMovement
        {
            get { return _AllowMovement; }
            set { _AllowMovement = value; }
        }

        public AABBox LineOfSite
        {
            get { return _LosNode.BoundingBoxTransformed; }
        }

        public Guard(IrrlichtDevice device, string contentPath)
            : base(device, contentPath)
        {
            _Waypath = new List<Waypoint>();

            AnimatedMesh mesh = _SceneMgr.GetMesh(contentPath + "ninja.b3d");
            _Node = _SceneMgr.AddAnimatedMeshSceneNode(mesh);

            _Node.SetMaterialTexture(0, _Driver.GetTexture(contentPath + "nskinrd.jpg"));

            _Node.Position = new Vector3Df(10f, 0.3485f, -100f);

            _Node.SetFrameLoop(0, 13); //13

            _Node.ID = 0;


            _LosNode = _SceneMgr.AddCubeSceneNode(5f, _Node, 0);
            _LosNode.Position = new Vector3Df(0f, 5f, 22f);
            _LosNode.Scale = new Vector3Df(1f, 1f, 10f);
            _LosNode.SetMaterialTexture(0, _Driver.GetTexture(contentPath + "invisible.png"));
            _LosNode.GetMaterial(0).Type = MaterialType.TransparentAlphaChannel;
            //_LosNode.DebugDataVisible = DebugSceneType.BBoxAll;
        }

        public override void Update()
        {
            this.FollowWaypath();
        }

        public void SetStartWaypath()
        {
            if (_Waypath != null && _Waypath.Count > 0)
            {
                // Snap the node to the start of the waypath
                _Node.Rotation = _Waypath[0].Rotation;
                _Node.Position = _Waypath[0].Position;
                _IsFollowingWaypath = true;
                _WaypathNode = 0;
                _WaypathInterval = 0;

                _Node.UpdateAbsolutePosition();
                _LosNode.UpdateAbsolutePosition();

                // Start following the waypath
                //SceneNodeAnimator anim = _SceneMgr.CreateFollowSplineAnimator(_Waypath, 0f, 0.5f, 0f, true, true);
                //_Node.AddAnimator(anim);
                //anim.Drop();
            }
        }

        public void FollowWaypath()
        {
            if (_IsFollowingWaypath && _AllowMovement)
            {
                int nextWaypathNode = (_WaypathNode + 1) % _Waypath.Count;
                Vector3Df posDiff = _Waypath[_WaypathNode].Position - _Waypath[nextWaypathNode].Position;
                Vector3Df posOffset = (posDiff / _AnimationSpeed).Invert();

                float rotDiff = _Waypath[_WaypathNode].Rotation.Y - _Waypath[nextWaypathNode].Rotation.Y;

                if (rotDiff > 180f) rotDiff = (rotDiff - 360f);

                float rotOffset = -(rotDiff / _AnimationSpeed);
                float rotY = _Node.Rotation.Y + rotOffset;

                if (rotY > 360f) rotY -= 360f;

                int lastInterval = _WaypathInterval;
                _WaypathInterval = (_WaypathInterval + 1) % (int)_AnimationSpeed;

                _Node.Rotation = new Vector3Df(_Node.Rotation.X, rotY, _Node.Rotation.Z);
                _Node.Position = new Vector3Df(_Node.Position.X + posOffset.X, _Node.Position.Y + posOffset.Y, _Node.Position.Z + posOffset.Z);

                if (lastInterval != 0 && _WaypathInterval == 0)
                    _WaypathNode = nextWaypathNode;
                /////////////////////////////////////////////////////////////////////////
            }
        }

        public void AddWaypoint(Vector3Df pos, Vector3Df rot)
        {
            _Waypath.Add(new Waypoint(pos, rot));
            SetStartWaypath();
        }

        public void RemoveLastWaypoint()
        {
            if (_Waypath.Count > 0)
            {
                _Waypath.RemoveAt(_Waypath.Count - 1);
                SetStartWaypath();
            }
        }

        public void LoadWaypathXml(string xmlFilePath)
        {
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load(xmlFilePath);

            XmlNodeList nodes = xmlDoc.GetElementsByTagName("Waypoint");

            foreach (XmlNode node in nodes)
            {
                try
                {
                    if (node.ChildNodes.Count >= 2)
                    {
                        XmlNode PosNode = node.ChildNodes[0];
                        XmlNode RotNode = node.ChildNodes[1];

                        float posX = float.Parse(PosNode.Attributes[0].Value);
                        float posY = float.Parse(PosNode.Attributes[1].Value);
                        float posZ = float.Parse(PosNode.Attributes[2].Value);
                        Vector3Df pos = new Vector3Df(posX, posY, posZ);

                        float rotX = float.Parse(RotNode.Attributes[0].Value);
                        float rotY = float.Parse(RotNode.Attributes[1].Value);
                        float rotZ = float.Parse(RotNode.Attributes[2].Value);
                        Vector3Df rot = new Vector3Df(rotX, rotY, rotZ);

                        Waypoint wayPoint = new Waypoint(pos, rot);
                        _Waypath.Add(wayPoint);
                    }
                }
                catch
                {
                    Console.WriteLine("Failed to load waypath xml data!");
                    break;
                }
            }
        }

        public void SaveWaypathXml(string xmlFilePath)
        {
            using (XmlWriter xmlDoc = XmlWriter.Create(xmlFilePath))
            {
                xmlDoc.WriteStartDocument();
                xmlDoc.WriteStartElement("Waypath");
                bool firstNode = true;

                foreach (Waypoint waypoint in _Waypath)
                {
                    if (firstNode)
                    {
                        firstNode = false;
                        continue;
                    }

                    xmlDoc.WriteStartElement("Waypoint");

                    xmlDoc.WriteStartElement("Position");
                    xmlDoc.WriteAttributeString("X", waypoint.Position.X.ToString());
                    xmlDoc.WriteAttributeString("Y", waypoint.Position.Y.ToString());
                    xmlDoc.WriteAttributeString("Z", waypoint.Position.Z.ToString());
                    xmlDoc.WriteEndElement();

                    xmlDoc.WriteStartElement("Rotation");
                    xmlDoc.WriteAttributeString("X", waypoint.Rotation.X.ToString());
                    xmlDoc.WriteAttributeString("Y", waypoint.Rotation.Y.ToString());
                    xmlDoc.WriteAttributeString("Z", waypoint.Rotation.Z.ToString());
                    xmlDoc.WriteEndElement();

                    xmlDoc.WriteEndElement();
                }

                xmlDoc.WriteEndDocument();
                xmlDoc.Close();
            }
        }
        ///////////////////////////////////////////////////////////////////////////////
    }
}
