﻿using System;
using System.Collections.Generic;
using System.Text;

using IrrlichtLime;
using IrrlichtLime.Core;
using IrrlichtLime.Video;
using IrrlichtLime.Scene;
using IrrlichtLime.GUI;
using System.Runtime.InteropServices;

namespace CapstoneProjectCSharp
{
    class Player : PlayerBase
    {
        const float playerAnimationSpeed = 24f;
        const float playerMoveSpeed = 0.5f;
        const float playerTurnSpeed = 2.5f;
        const float maxCamZoomOut = 20f;

        Vector3Df cameraFixedPos = new Vector3Df(0f, 15f, 15f); // 0, 15, 15
        CameraSceneNode _Camera;

        double camRotateX = 0f;
        float camRotateY = 0f;
        Vector3Df playerPos, cameraPos;

        private const int MaxTime = 5; // The time before the game is over in minutes
        private const float MaxStealRadius = 15f; // How close you need to be to an object to steal it
        private const int StealTime = 5; // The time it takes to steal an object in seconds
        private bool _IsStealing = false;
        private DateTime _TimeDoneStealing;
        private SceneNode _StealItem;
        private int _ObjectsTaken = 0;

        private DateTime _TimeStarted;
        private TimeSpan _TimeCompleted;
        private bool _GameOver = false;
        private bool _WasSpotted = false;
        private bool _OutOfTime = false;
        private bool _PlayerExited = false;
        private int _FadeOutInterval = 0;
        string _ContentPath = "";



        public bool GameOver
        {
            get { return _GameOver; }
            set { _GameOver = value; }
        }

        public bool WasSpotted
        {
            get { return _WasSpotted; }
            set { _WasSpotted = value; }
        }

        public bool OutOfTime
        {
            get { return _OutOfTime; }
            set { _OutOfTime = value; }
        }

        public bool PlayerExited
        {
            get { return _PlayerExited; }
            set { _PlayerExited = value; }
        }

        public bool IsStealing
        {
            get { return _IsStealing; }
            set { _IsStealing = value; }
        }

        public int ObjectsTaken
        {
            get { return _ObjectsTaken; }
            set { _ObjectsTaken = value; }
        }

        public Player(IrrlichtDevice device, string contentPath, Map map)
            : base(device, contentPath)
        {
            // Create the player
            AnimatedMesh playerMesh = _SceneMgr.GetMesh(contentPath + "ninja.b3d");
            _Node = _SceneMgr.AddAnimatedMeshSceneNode(playerMesh);
            _Node.SetMaterialTexture(0, _Driver.GetTexture(contentPath + "nskinbl.jpg"));
            //_Node.SetMaterialFlag(MaterialFlag.Lighting, false);
            _Node.Rotation = new Vector3Df(_Node.Rotation.X, _Node.Rotation.Y + 180f, _Node.Rotation.Z);
            //_Node.Position = new Vector3Df(0f, 0.3485f, 0f);
            _Node.Position = new Vector3Df(0f, 1.0f, 0f);
            //_Node.DebugDataVisible = DebugSceneType.BBox;
            _Node.ID = 0;


            // Create the camera
            _Camera = _SceneMgr.AddCameraSceneNode();
            _Camera.Target = new Vector3Df(0f);
            _Camera.Position = cameraFixedPos;


            // Create the player triangle selector for level collisions
            MeshSceneNode mapNode = map.SceneNode;
            TriangleSelector selector = null;

            if (mapNode != null)
            {
                //level.Position = new Vector3Df(-1350, -130, -1400);
                selector = _SceneMgr.CreateOctreeTriangleSelector(mapNode.Mesh, mapNode, 128); //128);
                mapNode.TriangleSelector = selector;

                if (selector != null)
                {
                    //SceneNodeAnimator anim0 = smgr.CreateCollisionResponseAnimator(
                    //    selector, _Camera,
                    //    new Vector3Df(1, 1, 1),
                    //    new Vector3Df(0, 0, 0),
                    //    new Vector3Df(0, 0, 0));

                    SceneNodeAnimator anim1 = _SceneMgr.CreateCollisionResponseAnimator(
                        selector, _Node,
                        new Vector3Df(1, 1, 1),
                        new Vector3Df(0, -10, 0),
                        new Vector3Df(0, 0, 0));

                    selector.Drop();
                    //_Camera.AddAnimator(anim0);
                    _Node.AddAnimator(anim1);
                    //anim0.Drop();
                    anim1.Drop();
                }
            }


            // Control variables
            playerPos = new Vector3Df(_Node.Position);
            cameraPos = (playerPos + cameraFixedPos);


            // Start the timer
            _TimeStarted = DateTime.Now.AddMinutes(MaxTime);


            // Other
            _ContentPath = contentPath;
            //////////////////////////////////////////////////////////////////////////////////////////
        }

        public void ResetPlayerWalkStatus()
        {
            _WasWalking = _IsWalking;
            _IsWalking = false;
            playerPos = new Vector3Df(_Node.Position);
        }

        public void MoveFoward()
        {
            // Move the player forward
            Vector3Df offset = new Vector3Df(playerPos.X, playerPos.Y, playerPos.Z + playerMoveSpeed);
            offset.RotateXZby(-_Node.Rotation.Y, playerPos);
            playerPos = offset;
            
            Vector3Df offset2 = new Vector3Df(cameraPos.X, cameraPos.Y, cameraPos.Z - playerMoveSpeed);
            offset2.RotateXZby(-_Camera.Rotation.Y, cameraPos);
            cameraPos = offset2;

            _IsStealing = false;
            _IsWalking = true;
            _MovementDirection = MovementDirections.Foward;
        }

        public void MoveBackward()
        {
            // Move the player backward
            Vector3Df offset = new Vector3Df(playerPos.X, playerPos.Y, playerPos.Z - playerMoveSpeed);
            offset.RotateXZby(-_Node.Rotation.Y, playerPos);
            playerPos = offset;
            
            Vector3Df offset2 = new Vector3Df(cameraPos.X, cameraPos.Y, cameraPos.Z + playerMoveSpeed);
            offset2.RotateXZby(-_Camera.Rotation.Y, cameraPos);
            cameraPos = offset2;

            _IsStealing = false;
            _IsWalking = true;
            _MovementDirection = MovementDirections.Bacward;
        }

        public void TurnLeft()
        {
            // Turn the player to the left
            _Node.Rotation = new Vector3Df(_Node.Rotation.X, _Node.Rotation.Y - playerTurnSpeed, _Node.Rotation.Z);
            _Camera.Rotation = new Vector3Df(_Camera.Rotation.X, _Camera.Rotation.Y - playerTurnSpeed, _Camera.Rotation.Z);
            //_Camera.Rotation = new Vector3Df(_Node.Rotation.X, _Node.Rotation.Y, _Node.Rotation.Z);
        }

        public void TurnRight()
        {
            // Turn the player to the right
            _Node.Rotation = new Vector3Df(_Node.Rotation.X, _Node.Rotation.Y + playerTurnSpeed, _Node.Rotation.Z);
            _Camera.Rotation = new Vector3Df(_Camera.Rotation.X, _Camera.Rotation.Y + playerTurnSpeed, _Camera.Rotation.Z);
            //_Camera.Rotation = new Vector3Df(_Node.Rotation.X, _Node.Rotation.Y, _Node.Rotation.Z);
        }

        public void OrbitCamera(Vector2Di mouseOffset)
        {
            Vector3Df camOffset = new Vector3Df(cameraPos);
            camRotateX = (double)mouseOffset.X;
            camRotateY = (float)mouseOffset.Y;
            camOffset.RotateXZby(camRotateX, playerPos);
            camOffset.Y += camRotateY * 0.1f;
            cameraPos = camOffset;
            //Console.WriteLine("Rotate Camera X: " + mouseOffset.X + " Y: " + mouseOffset.Y);
        }

        public void UpdatePosition()
        {
            // Update the player & camera position
            _Camera.Target = new Vector3Df(playerPos.X, playerPos.Y + 6f, playerPos.Z);
            _Node.Position = playerPos;

            // Prevent the camera from zooming out too much
            Vector3Df camBoundsOffset = cameraPos - playerPos;

            float camX = cameraPos.X, camY = cameraPos.Y, camZ = cameraPos.Z;
            if (!(camBoundsOffset.X < maxCamZoomOut && camBoundsOffset.X > -maxCamZoomOut))
                camX = playerPos.X + ((camBoundsOffset.X < maxCamZoomOut) ? -maxCamZoomOut : maxCamZoomOut);

            if (!(camBoundsOffset.Y < maxCamZoomOut && camBoundsOffset.Y > -maxCamZoomOut))
                camY = playerPos.Y + ((camBoundsOffset.Y < maxCamZoomOut) ? -maxCamZoomOut : maxCamZoomOut);

            if (!(camBoundsOffset.Z < maxCamZoomOut && camBoundsOffset.Z > -maxCamZoomOut))
                camZ = playerPos.Z + ((camBoundsOffset.Z < maxCamZoomOut) ? -maxCamZoomOut : maxCamZoomOut);

            _Camera.Position = cameraPos = new Vector3Df(camX, camY, camZ);
        }

        public void UpdateAnimation()
        {
            // Set the correct animation loops
            if (!_WasWalking && _IsWalking)
            {
                if (_MovementDirection == MovementDirections.Foward)
                {
                    _Node.SetFrameLoop(13, 0);
                    _Node.AnimationSpeed = playerAnimationSpeed;
                }
                else
                {
                    _Node.SetFrameLoop(0, 13);
                    _Node.AnimationSpeed = -playerAnimationSpeed;
                }
            }
            if (_WasWalking && !_IsWalking)
            {
                //Console.WriteLine("Stop the walk cycle loop");
                _Node.SetFrameLoop(205, 249); //250-299 //205-249 //183,204
                _WasWalking = false;
            }
        }

        public void ZoomCameraIn()
        {
            Vector3Df camOffset = (_Camera.Position - _Camera.Target) * 0.1f;
            Console.WriteLine("Zoom In!");
            cameraPos = _Camera.Position = 
                new Vector3Df(_Camera.Position.X + camOffset.X, _Camera.Position.Y + camOffset.Y, _Camera.Position.Z + camOffset.Z);
        }

        public void ZoomCameraOut()
        {
            Vector3Df camOffset = (_Camera.Position - _Camera.Target) * 0.1f;
            Console.WriteLine("Zoom Out!");
            cameraPos = _Camera.Position =
                new Vector3Df(_Camera.Position.X - camOffset.X, _Camera.Position.Y - camOffset.Y, _Camera.Position.Z - camOffset.Z);
        }

        public void ZoomCamera(ref int mouseWheelDelta)
        {
            if (mouseWheelDelta > 0)
                this.ZoomCameraIn();
            else
                this.ZoomCameraOut();
            mouseWheelDelta = 0;
        }
         
        public SceneNode GetSelectedNodeAtPoint(Vector2Di mousePoint)
        {
            Console.WriteLine("Get point at - X:" + mousePoint.X.ToString() + " Y:" + mousePoint.Y.ToString());
            Line3Df rayLine = _SceneMgr.SceneCollisionManager.GetRayFromScreenCoordinates(mousePoint, _Camera);
            return _SceneMgr.SceneCollisionManager.GetSceneNodeFromRayBB(rayLine, 1);
        }

        public void StealItem(SceneNode node)
        {
            if (!_IsStealing)
            {
                // Make sure we are close enough to the item
                float distanceFromObject = _Node.AbsolutePosition.GetDistanceFrom(node.AbsolutePosition);
                Console.WriteLine("Distance from Item: " + distanceFromObject.ToString());
                if (distanceFromObject <= MaxStealRadius)
                {
                    Console.WriteLine("Started Stealing Item: " + node.Name);
                    _TimeDoneStealing = DateTime.Now.AddSeconds(StealTime);
                    _StealItem = node;
                    _IsStealing = true;
                }
            }
        }

        public void UpdateStealStatus()
        {
            if (_IsStealing)
            {
                if (DateTime.Now >= _TimeDoneStealing)
                {
                    if (_StealItem != null)
                        _StealItem.Visible = false;

                    ++_ObjectsTaken;
                    _StealItem = null;
                    _IsStealing = false;
                    Console.WriteLine("Stole the object!");
                }
            }
        }

        public void UpdateGui()
        {
            int x, y;
            TimeSpan timeDiff;

            if (!_GameOver)
            {
                // Update the stealing progress bar... if we are stealing something
                if (_IsStealing && (DateTime.Now < _TimeDoneStealing))
                {
                    timeDiff = _TimeDoneStealing.Subtract(DateTime.Now);
                    double percent = (100.0 - ((timeDiff.TotalMilliseconds / (StealTime * 1000)) * 100.0));
                    x = (_Driver.ScreenSize.Width / 2) -50;
                    y = _Driver.ScreenSize.Height - 100;

                    // Backdrop
                    _Driver.Draw2DRectangle(
                        new Recti(x, y, x + 100, y + 20),
                        new IrrlichtLime.Video.Color(0, 0, 0, 128));

                    // Overlay
                    _Driver.Draw2DRectangle(
                        new Recti(x, y, x + (int)percent, y + 20),
                        new IrrlichtLime.Video.Color(255, 0, 0, 128));

                    // Text
                    _Device.GUIEnvironment.BuiltInFont.Draw(
                        "Stealing: " + _StealItem.Name,
                        new Vector2Di(x + 5, y + 5),
                        new IrrlichtLime.Video.Color(255, 255, 255));
                }

                // Objects Taken
                x = 20;
                y = _Driver.ScreenSize.Height - 50;

                _Driver.Draw2DRectangle(
                    new Recti(x, y, x + _Driver.ScreenSize.Width - 2 * x, y + 30),
                    new IrrlichtLime.Video.Color(0, 0, 0, 128));

                _Device.GUIEnvironment.BuiltInFont.Draw(
                    "Objects Taken: " + _ObjectsTaken.ToString(),
                    new Vector2Di(x + 5, y + 5),
                    new IrrlichtLime.Video.Color(255, 255, 255));

                // Time Left
                timeDiff = _TimeStarted.Subtract(DateTime.Now);
                if (timeDiff.TotalSeconds >= 0.0)
                {
                    string timeStr = ((int)(timeDiff.TotalSeconds / 60)).ToString() + ":"
                        + ((int)(timeDiff.TotalSeconds % 60)).ToString().PadLeft(2, '0');
                    IrrlichtLime.Video.Color textCol = (timeDiff.TotalSeconds <= 60.0)
                        ? new IrrlichtLime.Video.Color(255, 0, 0)
                        : new IrrlichtLime.Video.Color(255, 255, 255);
                    _Device.GUIEnvironment.BuiltInFont.Draw(
                        "Time Left: " + timeStr,
                        new Vector2Di(x + 5, y + 15),
                        textCol);
                }
            }

            // If the player was spotted... fade out to game over!
            if (_WasSpotted)
            {
                x = 0;
                y = 0;
                _Driver.Draw2DRectangle(
                    new Recti(x, y, _Driver.ViewPort.Width, _Driver.ViewPort.Height),
                    new IrrlichtLime.Video.Color(0, 0, 0, _FadeOutInterval));

                _Gui.AddImage(
                _Driver.GetTexture(_ContentPath + "GameOverText.png"),
                new Vector2Di((_Driver.ViewPort.Width / 2) - 256, (_Driver.ViewPort.Height / 2) - 128));
            }
            else
            {
                if (_GameOver)
                {
                    // Time Left
                    x = (_Driver.ScreenSize.Width / 2) - 50;
                    y = (_Driver.ScreenSize.Height / 2) - 100;

                    timeDiff = _TimeStarted.Subtract(DateTime.Now);

                    if (_TimeCompleted == TimeSpan.Zero)
                        _TimeCompleted = timeDiff;

                    string timeStr = ((int)(_TimeCompleted.TotalSeconds / 60)).ToString() + ":"
                        + ((int)(_TimeCompleted.TotalSeconds % 60)).ToString().PadLeft(2, '0');

                    double totalScore = (30.0 * _TimeCompleted.TotalSeconds) * _ObjectsTaken;

                    string msgStr = "You have successfully escaped from the museum\r\n"
                        + "You looted: " + _ObjectsTaken.ToString() + " Objects\r\n"
                        + "With: " + timeStr + " time remaining\r\n"
                        + "Total Score: " + totalScore.ToString("F0");

                    // Backdrop
                    _Driver.Draw2DRectangle(
                        new Recti(0, 0, _Driver.ViewPort.Width, _Driver.ViewPort.Height),
                        new IrrlichtLime.Video.Color(0, 0, 0, 200));

                    // Score Text
                    _Device.GUIEnvironment.BuiltInFont.Draw(
                    msgStr,
                    new Vector2Di(x - 50, y + 5),
                    new IrrlichtLime.Video.Color(255, 255, 255));
                }
            }
            ///////////////////////////////////////////////////////////////
        }

        public override void Update()
        {
            if (_GameOver) return;

            this.UpdatePosition();
            this.UpdateAnimation();
            this.UpdateStealStatus();

            // Is time up?
            if (_TimeStarted.Subtract(DateTime.Now).TotalSeconds <= 0.0)
            {
                // Game Over!
                _OutOfTime = true;
                _GameOver = true;
            }

            // Was the player spotted?
            if (_WasSpotted)
            {
                _FadeOutInterval = ((_FadeOutInterval + 5) > 255) ? 255 : (_FadeOutInterval + 5);

                if (_FadeOutInterval == 255)
                    _GameOver = true;
            }
        }
        ////////////////////////////////////////////////////////////////////////////
    }
}
