﻿using System;
using System.Collections.Generic;
using System.Text;

using IrrlichtLime;
using IrrlichtLime.Core;
using IrrlichtLime.Video;
using IrrlichtLime.Scene;
using IrrlichtLime.GUI;
using System.Runtime.InteropServices;

namespace CapstoneProjectCSharp
{
    public abstract class PlayerBase
    {
        public enum MovementDirections
        {
            Foward,
            Bacward
        }

        protected float _AnimationSpeed = 24f;
        protected float _MoveSpeed = 0.3f;
        protected float _TurnSpeed = 2.5f;
        protected bool _IsWalking = true;
        protected bool _WasWalking = false;
        protected AnimatedMeshSceneNode _Node;
        protected IrrlichtDevice _Device;
        protected VideoDriver _Driver;
        protected SceneManager _SceneMgr;
        protected GUIEnvironment _Gui;
        protected MovementDirections _MovementDirection;



        public AnimatedMeshSceneNode SceneNode
        {
            get { return _Node; }
        }

        public bool IsWalking
        {
            get { return _IsWalking; }
            set { _IsWalking = value; }
        }

        public bool WasWalking
        {
            get { return _WasWalking; }
            set { _WasWalking = value; }
        }

        public float AnimationSpeed
        {
            get { return _AnimationSpeed; }
            set { _AnimationSpeed = value; }
        }

        public float MoveSpeed
        {
            get { return _MoveSpeed; }
            set { _MoveSpeed = value; }
        }

        public float TurnSpeed
        {
            get { return _TurnSpeed; }
            set { _TurnSpeed = value; }
        }

        public PlayerBase(IrrlichtDevice device, string contentPath)
        {
            _Device = device;
            _Gui = device.GUIEnvironment;
            _Driver = device.VideoDriver;
            _SceneMgr = device.SceneManager;
        }

        public bool CheckBoundingBoxCollisionWith(AABBox b)
        {
            AABBox a = _Node.BoundingBoxTransformed;
            return (
                a.MaxEdge.X > b.MinEdge.X &&
                a.MinEdge.X < b.MaxEdge.X &&
                a.MaxEdge.Y > b.MinEdge.Y &&
                a.MinEdge.Y < b.MaxEdge.Y &&
                a.MaxEdge.Z > b.MinEdge.Z &&
                a.MinEdge.Z < b.MaxEdge.Z
            );
        }

        public abstract void Update();
    }
}
